@@ -111,7 +111,7 @@ void set_environment(t_session *session, t_fcgi_buffer *fcgi_buffer) {
 	size_t len1, len2, path_info_len;
 	bool has_path_info = false;
 	t_headerfield *headerfields;
-	t_keyvalue *envir;
+	t_keyvalue *envir, *header;
 #ifdef ENABLE_SSL
 	char subject[SSL_VAR_SIZE], issuer[SSL_VAR_SIZE];
 #endif
@@ -208,8 +208,10 @@ void set_environment(t_session *session, t_fcgi_buffer *fcgi_buffer) {
 	headerfield_to_environment(session, fcgi_buffer, "Accept-Charset:", "HTTP_ACCEPT_CHARSET");
 	headerfield_to_environment(session, fcgi_buffer, "Accept-Encoding:", "HTTP_ACCEPT_ENCODING");
 	headerfield_to_environment(session, fcgi_buffer, "Accept-Language:", "HTTP_ACCEPT_LANGUAGE");
+	headerfield_to_environment(session, fcgi_buffer, "Authorization:", "HTTP_AUTHORIZATION");
 	headerfield_to_environment(session, fcgi_buffer, "Client-IP:", "HTTP_CLIENT_IP");
 	headerfield_to_environment(session, fcgi_buffer, "DNT:", "HTTP_DNT");
+	headerfield_to_environment(session, fcgi_buffer, "Expect:", "HTTP_EXPECT");
 	headerfield_to_environment(session, fcgi_buffer, "From:", "HTTP_FROM");
 	headerfield_to_environment(session, fcgi_buffer, "Host:", "HTTP_HOST");
 	headerfield_to_environment(session, fcgi_buffer, "If-Modified-Since:", "HTTP_IF_MODIFIED_SINCE");
@@ -219,6 +221,15 @@ void set_environment(t_session *session, t_fcgi_buffer *fcgi_buffer) {
 	headerfield_to_environment(session, fcgi_buffer, "User-Agent:", "HTTP_USER_AGENT");
 	headerfield_to_environment(session, fcgi_buffer, "Via:", "HTTP_VIA");
 
+	/* Webdav headers
+	 */
+	if (session->host->webdav_app) {
+		headerfield_to_environment(session, fcgi_buffer, "Depth:", "HTTP_DEPTH");
+		headerfield_to_environment(session, fcgi_buffer, "Destination:", "HTTP_DESTINATION");
+		headerfield_to_environment(session, fcgi_buffer, "If:", "HTTP_IF");
+		headerfield_to_environment(session, fcgi_buffer, "Overwrite:", "HTTP_OVERWRITE");
+	}
+
 	/* Convert X-* HTTP headers to HTTP_* environment variables
 	 */
 	headerfields = session->headerfields;
