@@ -22,7 +22,6 @@
 #include <signal.h>
 #include <dirent.h>
 #include <errno.h>
-#include <limits.h>
 #include <pthread.h>
 #include <syslog.h>
 #include <grp.h>
@@ -45,6 +44,7 @@
 #include "userconfig.h"
 #include "session.h"
 #include "httpauth.h"
+#include "http.h"
 #include "send.h"
 #include "client.h"
 #include "target.h"
@@ -70,18 +70,13 @@
 #include "monitor.h"
 #endif
 
-#define REQUEST_BUFFER_CHUNK     4 * KILOBYTE
-
 #define rs_NONE                  0
 #define rs_QUIT_SERVER           1
 #define rs_UNBAN_CLIENTS         2
 #define rs_UNLOCK_LOGFILES       3
 #define rs_CLEAR_CACHE           4
 
-#define RANDOM_BUFFER_SIZE     512
-
-#define NO_REQUEST_LIMIT_TIME  300
-#define NO_REQUEST_LIMIT_SIZE   16 * MEGABYTE
+#define PTHREAD_STACK_SIZE 512 * KILOBYTE
 
 #define LOG_PERM (S_IRUSR|S_IWUSR|S_IRGRP)
 
@@ -96,6 +91,7 @@ bool must_quit = false;
 int open_connections = 0;
 
 char *hs_conlen      = "Content-Length: "; /* 16 */
+char *hs_forwarded   = "X-Forwarded-For:"; /* 16 */
 char *fb_filesystem  = "access denied via filesystem";
 char *fb_symlink     = "symlink not allowed";
 char *fb_accesslist  = "access denied via accesslist";
@@ -108,7 +104,7 @@ char *version_string = "Hiawatha v"VERSION
 	", chroot"
 #endif
 #ifdef ENABLE_COMMAND
-	", CommandChannel"
+	", Command Channel"
 #endif
 #ifdef ENABLE_DEBUG
 	", debug"
@@ -119,6 +115,9 @@ char *version_string = "Hiawatha v"VERSION
 #ifdef ENABLE_MONITOR
 	", Monitor"
 #endif
+#ifdef ENABLE_RPROXY
+	", reverse proxy"
+#endif
 #ifdef ENABLE_SSL
 	", SSL"
 #endif
@@ -346,444 +345,6 @@ int run_program(t_session *session, char *program, int return_code) {
 	return 0;
 }
 
-/* Read the request from a client socket.
- */
-int fetch_request(t_session *session) {
-	char *new_reqbuf, *strstart, *strend;
-	long max_request_size, bytes_read, header_length = -1, content_length = -1;
-	int result = 200, write_bytes, select_result;
-	time_t deadline;
-	fd_set read_fds;
-	struct timeval select_timeout;
-	bool keep_reading = true, store_on_disk = false;
-	int upload_handle = -1;
-
-	if (session->request_limit == false) {
-		deadline = session->time + NO_REQUEST_LIMIT_TIME;
-		max_request_size = NO_REQUEST_LIMIT_SIZE;
-	} else if (session->kept_alive == 0) {
-		deadline = session->time + session->binding->time_for_1st_request;
-		max_request_size = session->binding->max_request_size;
-	} else {
-		deadline = session->time + session->binding->time_for_request;
-		max_request_size = session->binding->max_request_size;
-	}
-
-	do {
-		/* Check if requestbuffer contains a complete request.
-		 */
-		if (session->request != NULL) {
-			if (header_length == -1) {
-				if ((strstart = strstr(session->request, "\r\n\r\n")) != NULL) {
-					*(strstart + 2) = '\0';
-					header_length = strstart + 4 - session->request;
-					session->header_length = header_length;
-
-					determine_request_method(session);
-					store_on_disk = (session->request_method == PUT) && session->binding->enable_alter;
-
-					if (store_on_disk) {
-	 					if ((session->uploaded_file = (char*)malloc(session->config->upload_directory_len + 15)) != NULL) {
-							strcpy(session->uploaded_file, session->config->upload_directory);
-							strcpy(session->uploaded_file + session->config->upload_directory_len, "/upload_XXXXXX");
-							if ((upload_handle = mkstemp(session->uploaded_file)) == -1) {
-								free(session->uploaded_file);
-								session->uploaded_file = NULL;
-							}
-						}
-						if (session->uploaded_file == NULL) {
-							log_error(session, "can't create temporary file for PUT request");
-							result = 500;
-							break;
-						}
-
-						session->uploaded_size = session->bytes_in_buffer - header_length;
-						if (write_buffer(upload_handle, session->request + header_length, session->uploaded_size) == -1) {
-							result = 500;
-							break;
-						}
-						session->bytes_in_buffer = header_length;
-					}
-
-				}
-			}
-			if (header_length != -1) {
-				if (content_length == -1) {
-					if ((strstart = strcasestr(session->request, hs_conlen)) != NULL) {
-						strstart += 16;
-						if ((strend = strstr(strstart, "\r\n")) != NULL) {
-							*strend = '\0';
-							content_length = str2int(strstart);
-							*strend = '\r';
-							if ((content_length < 0) || (INT_MAX - content_length - 2 <= header_length)) {
-								result = 500;
-								break;
-							}
-
-							if (store_on_disk) {
-								session->content_length = 0;
-								if (content_length > session->binding->max_upload_size) {
-									result = 413;
-									break;
-								}
-
-								session->buffer_size = header_length + REQUEST_BUFFER_CHUNK;
-								if ((new_reqbuf = (char*)realloc(session->request, session->buffer_size + 1)) != NULL) {
-									session->request = new_reqbuf;
-								} else {
-									session->error_cause = ec_SOCKET_READ_ERROR;
-									result = -1;
-									break;
-								}
-							} else {
-								session->content_length = content_length;
-								if (header_length + content_length > max_request_size) {
-									session->error_cause = ec_MAX_REQUESTSIZE;
-									result = -1;
-									break;
-								}
-
-								if (header_length + content_length > session->buffer_size) {
-									session->buffer_size = header_length + content_length;
-									if ((new_reqbuf = (char*)realloc(session->request, session->buffer_size + 1)) != NULL) {
-										session->request = new_reqbuf;
-									} else {
-										session->error_cause = ec_SOCKET_READ_ERROR;
-										result = -1;
-										break;
-									}
-								}
-							}
-						}
-					} else {
-						session->content_length = 0;
-						if (store_on_disk) {
-							result = 411;
-						}
-						break;
-					}
-				}
-
-				if (content_length > -1) {
-					if (store_on_disk) {
-						if (session->uploaded_size == content_length) {
-							/* Received a complete PUT request */
-							break;
-						}
-					} else {
-						if (session->bytes_in_buffer >= header_length + content_length) {
-							/* Received a complete request */
-							break;
-						}
-					}
-				}
-			}
-		}
-
-		FD_ZERO(&read_fds);
-		FD_SET(session->client_socket, &read_fds);
-
-		select_timeout.tv_sec = 1;
-		select_timeout.tv_usec = 0;
-
-#ifdef ENABLE_SSL
-		select_result = session->binding->use_ssl ? ssl_pending(&(session->ssl_data)) : 0;
-
-		if (select_result == 0)
-#endif
-			select_result = select(session->client_socket + 1, &read_fds, NULL, NULL, &select_timeout);
-
-		switch (select_result) {
-			case -1:
-				if (errno != EINTR) {
-					session->error_cause = ec_SOCKET_READ_ERROR;
-					result = -1;
-					keep_reading = false;
-				}
-				break;
-			case 0:
-				if (session->force_quit) {
-					session->error_cause = ec_FORCE_QUIT;
-					result = -1;
-					keep_reading = false;
-				} else if (time(NULL) > deadline) {
-					session->error_cause = ec_TIMEOUT;
-					result = -1;
-					keep_reading = false;
-				}
-				break;
-			default:
-				if ((content_length == -1) && ((session->buffer_size - session->bytes_in_buffer) < 256)) {
-					session->buffer_size += REQUEST_BUFFER_CHUNK;
-					if ((new_reqbuf = (char*)realloc(session->request, session->buffer_size + 1)) != NULL) {
-						session->request = new_reqbuf;
-					} else {
-						session->error_cause = ec_SOCKET_READ_ERROR;
-						result = -1;
-						keep_reading = false;
-						break;
-					}
-				}
-
-				/* Read from socket.
-				 */
-#ifdef ENABLE_SSL
-				if (session->binding->use_ssl) {
-					bytes_read = ssl_receive(&(session->ssl_data), session->request + session->bytes_in_buffer, 
-									session->buffer_size - session->bytes_in_buffer);
-				} else
-#endif
-					bytes_read = recv(session->client_socket, session->request + session->bytes_in_buffer,
-									session->buffer_size - session->bytes_in_buffer, 0);
-
-				switch (bytes_read) {
-					case -1:
-						if (errno != EINTR) {
-							session->error_cause = ec_SOCKET_READ_ERROR;
-							result = -1;
-							keep_reading = false;
-						}
-						break;
-					case 0:
-						session->error_cause = ec_CLIENT_DISCONNECTED;
-						result = -1;
-						keep_reading = false;
-						break;
-					default:
-						if (store_on_disk) {
-							/* Write to file on disk
-							 */
-							write_bytes = bytes_read;
-							if (session->uploaded_size + bytes_read > content_length) {
-								write_bytes -= ((session->uploaded_size + bytes_read) - content_length);
-							}
-							if (write_buffer(upload_handle, session->request + header_length, write_bytes) == -1) {
-								result = 500;
-								keep_reading = false;
-								break;
-							}
-							if ((session->uploaded_size += write_bytes) > session->binding->max_upload_size) {
-								result = 413;
-								keep_reading = false;
-								break;
-							}
-							if (write_bytes < bytes_read) {
-								memmove(session->request + header_length, session->request + header_length + write_bytes, bytes_read - write_bytes);
-								session->bytes_in_buffer += bytes_read - write_bytes;
-								keep_reading = false;
-							}
-						} else {
-							/* Read into memory
-							 */
-							session->bytes_in_buffer += bytes_read;
-							*(session->request + session->bytes_in_buffer) = '\0';
-
-							if (session->bytes_in_buffer > max_request_size) {
-								keep_reading = false;
-								session->error_cause = ec_MAX_REQUESTSIZE;
-								result = -1;
-								break;
-							}
-						}
-				}
-		}
-	} while (keep_reading);
-
-	if (upload_handle != -1) {
-		fsync(upload_handle);
-		close(upload_handle);
-	}
-
-#ifdef ENABLE_COMMAND
-	increment_transfer(TRANSFER_RECEIVED, header_length + content_length);
-#endif
-
-	return result;
-}
-
-/* Convert the requestbuffer to a session record.
- */
-int parse_request(t_session *session, int total_bytes) {
-	int retval = 200;
-	char *request_end, *str_end, *conn;
-
-	request_end = session->request + total_bytes;
-
-	/* Request method
-	 */
-	session->method = str_end = session->request;
-	while ((*str_end != ' ') && (str_end != request_end)) {
-		str_end++;
-	}
-	if (str_end == request_end) {
-		return 400;
-	}
-	*str_end = '\0';
-	session->uri = ++str_end;
-
-	/* URI
-	 */
-	while ((*str_end != ' ') && (str_end != request_end)) {
-		str_end++;
-	}
-	if (str_end == request_end) {
-		return 400;
-	}
-	*(str_end++) = '\0';
-	session->uri_len = strlen(session->uri);
-	if ((session->config->max_url_length > 0) && (session->uri_len > session->config->max_url_length)) {
-		return 414;
-	}
-
-	if (strncmp(session->uri, "http://", 7) == 0) {
-		return 400;
-	} else if ((session->request_uri = strdup(session->uri)) == NULL) {
-		return -1;
-	}
-
-	/* Protocol version
-	 */
-	if (min_strlen(str_end, 10) == false) {
-		return 400;
-	} else if (memcmp(str_end, "HTTP/", 5) != 0) {
-		return 400;
-	}
-
-	session->http_version = str_end;
-	str_end += 7;
-
-	if ((*(str_end - 1) != '.') || (*(str_end + 1) != '\r') || (*(str_end + 2) != '\n')) {
-		return 400;
-	} else if (*(str_end - 2) != '1') {
-		return 505;
-	}
-	*(str_end + 1) = '\0';
-
-	/* Body and other request headerlines
-	 */
-	if (session->content_length > 0) {
-		session->body = session->request + session->header_length;
-	}
-	session->headerfields = parse_headerfields(str_end + 3);
-	session->hostname = strlower(get_headerfield("Host:", session->headerfields));
-	session->cookie = get_headerfield("Cookie:", session->headerfields);
-
-	if ((conn = get_headerfield("Connection:", session->headerfields)) != NULL) {
-		conn = strlower(remove_spaces(conn));
-	}
-	session->keep_alive = false;
-
-	switch (*str_end) {
-		case '0':
-			if ((conn != NULL) && (session->kept_alive < session->binding->max_keepalive)) {
-				if (strcasecmp(conn, "keep-alive") == 0) {
-					session->keep_alive = true;
-				}
-			}
-			break;
-		case '1':
-			if (session->hostname == NULL) {
-				retval = 400;
-			} else if (session->kept_alive < session->binding->max_keepalive) {
-				session->keep_alive = true;
-				if (conn != NULL) {
-					if (strcmp(conn, "close") == 0) {
-						session->keep_alive = false;
-					}
-				}
-			}
-			break;
-		default:
-			retval = 505;
-			break;
-	}
-	if (session->keep_alive) {
-		session->kept_alive++;
-	}
-
-	return retval;
-}
-
-/* Convert the request uri to a filename.
- */
-int uri_to_path(t_session *session) {
-	size_t length, alias_length = 0;
-	char *strstart, *strend;
-	t_keyvalue *alias;
-	int retval;
-
-	/* Requested file in userdirectory?
-	 */
-	if (session->host->user_websites && (session->uri_len >= 3)) {
-		if (*(session->uri + 1) == '~') {
-			strstart = session->uri + 1;
-			if ((strend = strchr(strstart, '/')) == NULL) {
-				return 301;
-			} else if ((length = strend - strstart) > 1) {
-				if ((session->local_user = (char*)malloc(length + 1)) == NULL) {
-					return 500;
-				}
-
-				memcpy(session->local_user, strstart, length);
-				*(session->local_user + length) = '\0';
-
-				if ((retval = get_homedir(session, session->local_user + 1)) != 200) {
-					return retval;
-				}
-				session->host->error_handlers = NULL;
-			} else {
-				/* uri is '/~/...' */
-				return 404;
-			}
-		}
-	}
-
-	/* Search for an alias.
-	 */
-	alias = session->host->alias;
-	while (alias != NULL) {
-		alias_length = strlen(alias->key);
-		if (strncmp(session->uri, alias->key, alias_length) == 0) {
-			if ((*(session->uri + alias_length) == '/') || (*(session->uri + alias_length) == '\0')) {
-				session->alias_used = true;
-				break;
-			}
-		}
-		alias = alias->next;
-	}
-
-	/* Allocate memory
-	 */
-	if (alias == NULL) {
-		length = session->host->website_root_len;
-	} else {
-		length = strlen(alias->value);
-	}
-	length += session->uri_len + MAX_START_FILE_LENGTH;
-	if ((session->file_on_disk = (char*)malloc(length + 4)) == NULL) { /* + 3 for '.gz' (gzip encoding) */
-		return 500;
-	}
-
-	/* Copy stuff
-	 */
-	if (alias == NULL) {
-		length = session->host->website_root_len;
-		memcpy(session->file_on_disk, session->host->website_root, length);
-		strstart = session->uri;
-		if (session->local_user != NULL) {
-			strstart += strlen(session->local_user) + 1;
-		}
-	} else {
-		length = strlen(alias->value);
-		memcpy(session->file_on_disk, alias->value, length);
-		strstart = session->uri + alias_length;
-
-	}
-	strcpy(session->file_on_disk + length, strstart);
-
-	return 200;
-}
-
 t_access allow_client(t_session *session) {
 	char *x_forwarded_for;
 	t_ip_addr forwarded_ip;
@@ -791,7 +352,7 @@ t_access allow_client(t_session *session) {
 
 	if ((access = ip_allowed(&(session->ip_address), session->host->access_list)) != allow) {
 		return access;
-	} else if ((x_forwarded_for = get_headerfield("X-Forwarded-For:", session->headerfields)) == NULL) {
+	} else if ((x_forwarded_for = get_headerfield(hs_forwarded, session->headerfields)) == NULL) {
 		return allow;
 	} else if (parse_ip(x_forwarded_for, &forwarded_ip) == -1) {
 		return allow;
@@ -802,71 +363,6 @@ t_access allow_client(t_session *session) {
 	return unspecified;
 }
 
-int get_path_info(t_session *session) {
-	t_fsbool is_dir;
-	char *slash;
-
-	if (session->alias_used) {
-		return 200;
-	}
-
-	if (session->host->website_root_len >= strlen(session->file_on_disk)) {
-		return 500;
-	}
-
-	slash = session->file_on_disk + session->host->website_root_len + 1;
-	while (*slash != '\0') {
-		if (*slash == '/') {
-			*slash = '\0';
-			is_dir = is_directory(session->file_on_disk);
-			*slash = '/';
-
-			switch (is_dir) {
-				case error:
-					return 500;
-				case not_found:
-					return 404;
-				case no_access:
-					return 403;
-				case no:
-					if ((session->path_info = strdup(slash)) == NULL) {
-						return -1;
-					}
-					*slash = '\0';
-					return 200;
-				case yes:
-					break;
-			}
-		}
-		slash++;
-	}
-
-	return 200;
-}
-
-/* Validate URL
- */
-bool validate_url(t_session *session) {
-	if (valid_uri(session->uri, session->host->allow_dot_files)) {
-		return true;
-	}
-
-	log_exploit_attempt(session, "invalid URL", NULL);
-#ifdef ENABLE_COMMAND
-	increment_counter(COUNTER_EXPLOIT);
-#endif
-#ifdef ENABLE_MONITOR
-	if (session->config->monitor_enabled) {
-		monitor_counter_exploit_attempt(session);
-	}
-#endif
-
-	session->return_code = (session->request_method == PUT) ? 403 : 404;
-	session->error_cause = ec_INVALID_URL;
-
-	return false;
-}
-
 /* Serve the client that connected to the webserver
  */
 int serve_client(t_session *session) {
@@ -876,11 +372,14 @@ int serve_client(t_session *session) {
 	t_access access;
 	t_deny_body *deny_body;
 	t_req_method request_method;
+	t_ip_addr ip;
 #ifdef ENABLE_TOOLKIT
 	int i;
-	t_toolkit_options options;
+	t_toolkit_options toolkit_options;
+#endif
+#ifdef ENABLE_RPROXY
+	t_rproxy *rproxy;
 #endif
-	t_ip_addr ip;
 
 	if ((result = fetch_request(session)) != 200) {
 		session->request_method = GET;
@@ -895,7 +394,7 @@ int serve_client(t_session *session) {
 	/* Hide reverse proxies
 	 */
 	if (in_iplist(session->config->hide_proxy, &(session->ip_address))) {
-		if ((client_ip = get_headerfield("X-Forwarded-For:", session->headerfields)) != NULL) {
+		if ((client_ip = get_headerfield(hs_forwarded, session->headerfields)) != NULL) {
 			if ((search = strrchr(client_ip, ',')) != NULL) {
 				client_ip = search + 1;
 			}
@@ -928,35 +427,15 @@ int serve_client(t_session *session) {
 	}
 	session->host->access_time = session->time;
 
-	/* Actions based on request method
+	/* Reject bots
 	 */
-	switch (session->request_method) {
-		case TRACE:
-			if (session->binding->enable_trace == false) {
-				return 501;
-			}
-			return handle_trace_request(session);
-		case PUT:
-		case DELETE:
-			if ((session->binding->enable_alter == false) && (session->host->webdav_app == false)) {
-				return 501;
-			}
-			break;
-		case unknown:
-			return 400;
-		case unsupported:
-			if (session->host->webdav_app == false) {
-				return 501;
-			}
-			break;
+	if (client_is_rejected_bot(session)) {
+		log_error(session, "bot rejected");
+		return 403;
 	}
 
-	/* Validate URL
+	/* Enforce usage of SSL
 	 */
-	if (validate_url(session) == false) {
-		return -1;
-	}
-
 #ifdef ENABLE_SSL
 	if (session->host->require_ssl && (session->binding->use_ssl == false)) {
 		if ((qmark = strchr(session->uri, '?')) != NULL) {
@@ -969,22 +448,6 @@ int serve_client(t_session *session) {
 	}
 #endif
 
-	if (session->host->secure_url) {
-		if (strstr(session->request_uri, "%00") != NULL) {
-			log_exploit_attempt(session, "NULL-byte in URI", NULL);
-#ifdef ENABLE_COMMAND
-			increment_counter(COUNTER_EXPLOIT);
-#endif
-#ifdef ENABLE_MONITOR
-			if (session->config->monitor_enabled) {
-				monitor_counter_exploit_attempt(session);
-			}
-#endif
-
-			return 403;
-		}
-	}
-
 	/* Deny matching bodies
 	 */
 	if (session->body != NULL) {
@@ -1024,25 +487,75 @@ int serve_client(t_session *session) {
 		*(session->body + session->content_length) = chr;
 	}
 
+#ifdef ENABLE_RPROXY
+	/* Reverse proxy
+	 */
+	rproxy = session->host->rproxy;
+	while (rproxy != NULL) {
+		if (rproxy_match(rproxy, session->request_uri)) {
+			if (validate_url(session) == false) {
+				return -1;
+			}
+
+			if ((session->vars != NULL) && (session->host->secure_url)) {
+				if (forbidden_chars_present(session->vars)) {
+					return 403;
+				}
+			}
+
+			if (rproxy_loop_detected(session->headerfields)) {
+				return 508;
+			}
+
+			return proxy_request(session, rproxy);
+		}
+
+		rproxy = rproxy->next;
+	}
+#endif
+
+	/* Actions based on request method
+	 */
+	switch (session->request_method) {
+		case TRACE:
+			if (session->binding->enable_trace == false) {
+				return 501;
+			}
+			return handle_trace_request(session);
+		case PUT:
+		case DELETE:
+			if ((session->binding->enable_alter == false) && (session->host->webdav_app == false)) {
+				return 501;
+			}
+			break;
+		case unknown:
+			return 400;
+		case unsupported:
+			if (session->host->webdav_app == false) {
+				return 501;
+			}
+			break;
+	}
+
 #ifdef ENABLE_TOOLKIT
 	/* URL toolkit
 	 */
 #ifdef ENABLE_SSL
-	init_toolkit_options(&options, session->host->website_root, session->config->url_toolkit,
+	init_toolkit_options(&toolkit_options, session->host->website_root, session->config->url_toolkit,
 	                     session->binding->use_ssl, session->host->allow_dot_files, session->headerfields);
 #else
-	init_toolkit_options(&options, session->host->website_root, session->config->url_toolkit,
+	init_toolkit_options(&toolkit_options, session->host->website_root, session->config->url_toolkit,
 	                     false, session->host->allow_dot_files, session->headerfields);
 #endif
 
 	if ((session->request_method != PUT) && (session->request_method != DELETE)) {
 		for (i = 0; i < session->host->toolkit_rules.size; i++) {
-			if ((result = use_toolkit(session->uri, session->host->toolkit_rules.item[i], &options)) == UT_ERROR) {
+			if ((result = use_toolkit(session->uri, session->host->toolkit_rules.item[i], &toolkit_options)) == UT_ERROR) {
 				return 500;
 			}
 
-			if ((options.ban > 0) && (ip_allowed(&(session->ip_address), session->config->banlist_mask) != deny)) {
-				ban_ip(&(session->ip_address), options.ban, session->config->kick_on_ban);
+			if ((toolkit_options.ban > 0) && (ip_allowed(&(session->ip_address), session->config->banlist_mask) != deny)) {
+				ban_ip(&(session->ip_address), toolkit_options.ban, session->config->kick_on_ban);
 				log_system(session, "Client banned because of URL match in UrlToolkit rule");
 #ifdef ENABLE_MONITOR
 				if (session->config->monitor_enabled) {
@@ -1052,17 +565,17 @@ int serve_client(t_session *session) {
 				return 403;
 			}
 
-			session->toolkit_fastcgi = options.fastcgi_server;
-			if (options.new_url != NULL) {
-				if (register_tempdata(&(session->tempdata), options.new_url, tc_data) == -1) {
-					free(options.new_url);
+			session->toolkit_fastcgi = toolkit_options.fastcgi_server;
+			if (toolkit_options.new_url != NULL) {
+				if (register_tempdata(&(session->tempdata), toolkit_options.new_url, tc_data) == -1) {
+					free(toolkit_options.new_url);
 					return 500;
 				}
-				session->uri = options.new_url;
+				session->uri = toolkit_options.new_url;
 			}
 
 			if (result == UT_REDIRECT) {
-				session->location = strdup(options.new_url);
+				session->location = strdup(toolkit_options.new_url);
 				session->cause_of_301 = location;
 				return 301;
 			}
@@ -1072,8 +585,8 @@ int serve_client(t_session *session) {
 				return 403;
 			}
 
-			if (options.expire > -1) {
-				session->expires = options.expire;
+			if (toolkit_options.expire > -1) {
+				session->expires = toolkit_options.expire;
 			}
 		}
 	}
@@ -1134,11 +647,6 @@ int serve_client(t_session *session) {
 		}
 	}
 
-	if (client_is_rejected_bot(session)) {
-		log_error(session, "bot rejected");
-		return 403;
-	}
-
 	if ((result = copy_directory_settings(session)) != 200) {
 		return result;
 	}
@@ -1532,7 +1040,8 @@ void connection_handler(t_session *session) {
 		remove_client(session, true);
 	}
 
-	/* Client session ends here */
+	/* Client session ends here
+	 */
 	pthread_exit(NULL);
 }
 
@@ -2206,6 +1715,12 @@ int run_server(t_settings *settings) {
 #endif
 #ifdef ENABLE_XSLT
 	init_xslt_module();
+#endif
+#ifdef ENABLE_RPROXY
+	if (init_rproxy_module() == -1) {
+		fprintf(stderr, "Error initializing reverse proxy module.\n");
+		return -1;
+	}
 #endif
 	if (init_sqli_detection() == -1) {
 		fprintf(stderr, "Error initializing SQL injection detection.\n");
@@ -2214,7 +1729,7 @@ int run_server(t_settings *settings) {
 #ifdef ENABLE_MONITOR
 	if (config->monitor_enabled) {
 		if (init_monitor_module(config) == -1) {
-			fprintf(stderr, "Monitor initialization failure.\n");
+			fprintf(stderr, "Error initializing Monitor module.\n");
 			return -1;
 		}
 		monitor_server_start();
