@@ -509,6 +509,27 @@ void remove_filelist(t_filelist *filelist) {
 	}
 }
 
+/* Send buffer to handle
+ */
+int write_buffer(int handle, const char *buffer, int size) {
+	long total_written = 0;
+	ssize_t bytes_written;
+
+	if (size <= 0) {
+		return 0;
+	} else while (total_written < size) {
+		if ((bytes_written = write(handle, buffer + total_written, size - total_written)) == -1) {
+			if (errno != EINTR) {
+				return -1;
+			}
+		} else {
+			total_written += bytes_written;
+		}
+	}
+
+	return 0;
+}
+
 #ifdef CYGWIN
 char *cygwin_to_windows(char *path) {
 	char *slash;
