@@ -265,7 +265,7 @@ int ssl_receive(ssl_context *ssl, char *buffer, unsigned int maxlength) {
 
 /* Send data via SSL connection
  */
-int ssl_send(ssl_context *ssl, char *buffer, unsigned int length) {
+int ssl_send(ssl_context *ssl, const char *buffer, unsigned int length) {
 	int result;
 
 	do {
@@ -312,4 +312,45 @@ void ssl_shutdown(void) {
 	rsa_free(&rsa);
 }
 
+#ifdef ENABLE_RPROXY
+int ssl_connect(ssl_context *ssl, ssl_session *ssn, int *sock) {
+	memset(ssn, 0, sizeof(ssl_session));
+	memset(ssl, 0, sizeof(ssl_context));
+
+	if (ssl_init(ssl) != 0) {
+		return -1;
+	}
+
+	ssl_set_endpoint(ssl, SSL_IS_CLIENT);
+	ssl_set_authmode(ssl, SSL_VERIFY_NONE);
+
+	ssl_set_rng(ssl, ssl_random, &ctr_drbg);
+	ssl_set_dbg(ssl, ssl_debug, stderr);
+	ssl_set_bio(ssl, net_recv, sock, net_send, sock);
+
+	ssl_set_ciphersuites(ssl, ciphersuites);
+	ssl_set_session(ssl, 0, 0, ssn);
+
+	if (ssl_handshake(ssl) != 0) {
+		return -1;
+	}
+
+	return 0;
+}
+
+int ssl_send_completely(ssl_context *ssl, const char *buffer, int size) {
+	int bytes_written, total_written = 0;
+
+	while (total_written < size) {
+		if ((bytes_written = ssl_write(ssl, buffer + total_written, size - total_written)) == -1) {
+			return -1;
+		} else {
+			total_written += bytes_written;
+		}
+	}
+
+	return total_written;
+}
+#endif
+
 #endif
