@@ -22,7 +22,7 @@ void log_string(char *logfile, char *mesg, ...);
 void log_system(t_session *session, char *mesg, ...);
 void log_file_error(t_session *session, char *file, char *mesg);
 void log_error(t_session *session, char *mesg);
-void log_request(t_session *session, int code);
+void log_request(t_session *session);
 void log_garbage(t_session *session);
 void log_exploit_attempt(t_session *session, char *type, char *data);
 void log_unban(char *logfile, t_ip_addr *ip_address, unsigned long connect_attempts);
