@@ -23,7 +23,9 @@
 #ifdef ENABLE_SSL
 #include "libssl.h"
 #endif
+#include "libfs.h"
 #include "libstr.h"
+#include "http.h"
 #include "log.h"
 #include "send.h"
 #ifdef ENABLE_COMMAND
@@ -526,26 +528,6 @@ int send_code(t_session *session) {
 	return 0;
 }
 
-/* Send directly to socket, unbuffered
- */
-int send_directly(int sock, const char *buffer, int size) {
-	int total_sent = 0, bytes_sent;
-
-	if (size <= 0) {
-		return 0;
-	} else while (total_sent < size) {
-		if ((bytes_sent = send(sock, buffer + total_sent, size - total_sent, 0)) == -1) {
-			if (errno != EINTR) {
-				return -1;
-			}
-		} else {
-			total_sent += bytes_sent;
-		}
-	}
-
-	return 0;
-}
-
 static int set_padding(t_fcgi_buffer *fcgi_buffer, bool adjust_buffer) {
 	unsigned char padding;
 
@@ -567,7 +549,7 @@ int send_fcgi_buffer(t_fcgi_buffer *fcgi_buffer, const char *buffer, int size) {
 	if (size > FCGI_BUFFER_SIZE) {
 		if (fcgi_buffer->size > 0) {
 			set_padding(fcgi_buffer, true);
-			if (send_directly(fcgi_buffer->sock, (char*)fcgi_buffer->data, fcgi_buffer->size) == -1) {
+			if (write_buffer(fcgi_buffer->sock, (char*)fcgi_buffer->data, fcgi_buffer->size) == -1) {
 				return -1;
 			}
 		}
@@ -578,16 +560,16 @@ int send_fcgi_buffer(t_fcgi_buffer *fcgi_buffer, const char *buffer, int size) {
 		fcgi_buffer->data[5] = FCGI_BUFFER_SIZE & 255;
 
 		padding = set_padding(fcgi_buffer, false);
-		if (send_directly(fcgi_buffer->sock, (char*)fcgi_buffer->data, 8) == -1) {
+		if (write_buffer(fcgi_buffer->sock, (char*)fcgi_buffer->data, 8) == -1) {
 			return -1;
-		} else if (send_directly(fcgi_buffer->sock, buffer, FCGI_BUFFER_SIZE) == -1) {
+		} else if (write_buffer(fcgi_buffer->sock, buffer, FCGI_BUFFER_SIZE) == -1) {
 			return -1;
 		}
 
 		fcgi_buffer->size = 0;
 		memset(fcgi_buffer->data, 0, (size_t)padding);
 
-		if (send_directly(fcgi_buffer->sock, (char*)fcgi_buffer->data, padding) == -1) {
+		if (write_buffer(fcgi_buffer->sock, (char*)fcgi_buffer->data, padding) == -1) {
 			return -1;
 		} else if (send_fcgi_buffer(fcgi_buffer, buffer + FCGI_BUFFER_SIZE, size - FCGI_BUFFER_SIZE) == -1) {
 			return -1;
@@ -595,7 +577,7 @@ int send_fcgi_buffer(t_fcgi_buffer *fcgi_buffer, const char *buffer, int size) {
 	} else if (buffer == NULL) {
 		if (fcgi_buffer->size > 0) {
 			set_padding(fcgi_buffer, true);
-			if (send_directly(fcgi_buffer->sock, (char*)fcgi_buffer->data, fcgi_buffer->size) == -1) {
+			if (write_buffer(fcgi_buffer->sock, (char*)fcgi_buffer->data, fcgi_buffer->size) == -1) {
 				return -1;
 			}
 		}
@@ -603,7 +585,7 @@ int send_fcgi_buffer(t_fcgi_buffer *fcgi_buffer, const char *buffer, int size) {
 		memcpy(fcgi_buffer->data, "\x01\x00\x00\x01" "\x00\x00\x00\x00", 8);
 		fcgi_buffer->data[1] = fcgi_buffer->mode;
 		set_padding(fcgi_buffer, true);
-		if (send_directly(fcgi_buffer->sock, (char*)fcgi_buffer->data, 8) == -1) {
+		if (write_buffer(fcgi_buffer->sock, (char*)fcgi_buffer->data, 8) == -1) {
 			return -1;
 		}
 
@@ -611,7 +593,7 @@ int send_fcgi_buffer(t_fcgi_buffer *fcgi_buffer, const char *buffer, int size) {
 	} else {
 		if ((fcgi_buffer->size + size > FCGI_BUFFER_SIZE) && (fcgi_buffer->size > 0)) {
 			set_padding(fcgi_buffer, true);
-			if (send_directly(fcgi_buffer->sock, (char*)fcgi_buffer->data, fcgi_buffer->size) == -1) {
+			if (write_buffer(fcgi_buffer->sock, (char*)fcgi_buffer->data, fcgi_buffer->size) == -1) {
 				return -1;
 			}
 			fcgi_buffer->size = 0;
@@ -676,22 +658,3 @@ void send_digest_auth(t_session *session) {
 	}
 	send_buffer(session, "\", algorithm=MD5, stale=false\r\n", 31);
 }
-
-int write_buffer(int handle, const char *buffer, int size) {
-	long total_written = 0;
-	ssize_t bytes_written;
-
-	if (size <= 0) {
-		return 0;
-	} else while (total_written < size) {
-		if ((bytes_written = write(handle, buffer + total_written, size - total_written)) == -1) {
-			if (errno != EINTR) {
-				return -1;
-			}
-		} else {
-			total_written += bytes_written;
-		}
-	}
-
-	return 0;
-}
