@@ -97,6 +97,9 @@ static t_host *new_host(void) {
 #endif
 #ifdef ENABLE_XSLT
 	host->use_xslt           = false;
+#endif
+#ifdef ENABLE_RPROXY
+	host->rproxy             = NULL;
 #endif
 	host->prevent_sqli       = false;
 	host->prevent_xss        = false;
@@ -1129,6 +1132,9 @@ static bool host_setting(char *key, char *value, t_host *host) {
 	char *botname;
 	t_denybotlist *deny_bot;
 	t_deny_body *deny_body;
+#ifdef ENABLE_RPROXY
+	t_rproxy *rproxy, *list;
+#endif
 
 	if (strcmp(key, "accesslogfile") == 0) {
 		if (*value == '/') {
@@ -1240,6 +1246,22 @@ static bool host_setting(char *key, char *value, t_host *host) {
 		if (parse_charlist(value, &(host->required_binding)) == 0) {
 			return true;
 		}
+#ifdef ENABLE_RPROXY
+	} else if (strcmp(key, "reverseproxy") == 0) {
+		if ((rproxy = rproxy_setting(value)) != NULL) {
+			if (host->rproxy == NULL) {
+				host->rproxy = rproxy;
+				return true;
+			} else {
+				list = host->rproxy;
+				while (list->next != NULL) {
+					list = list->next;
+				}
+				list->next = rproxy;
+				return true;
+			}
+		}
+#endif
 	} else if (strcmp(key, "secureurl") == 0) {
 		if (parse_yesno(value, &(host->secure_url)) == 0) {
 			return true;
@@ -1531,7 +1553,7 @@ static bool fcgi_server_setting(char *key, char *value, t_fcgi_server *fcgi_serv
 				}
 			} else {
 				connect_to->unix_socket = NULL;
-				if (parse_ip_port(value, &(connect_to->host), &(connect_to->port)) == -1) {
+				if (parse_ip_port(value, &(connect_to->ip_addr), &(connect_to->port)) == -1) {
 					return false;
 				}
 			}
