@@ -34,7 +34,7 @@
 #define ec_INVALID_URL          -8
 
 typedef enum { no_cgi, binary, script, fastcgi} t_cgi_type;
-typedef enum { unknown, GET, POST, HEAD, OPTIONS, TRACE, PUT, DELETE, unsupported } t_req_method;
+typedef enum { unknown, GET, POST, HEAD, TRACE, PUT, DELETE, unsupported } t_req_method;
 typedef enum { missing_slash, require_ssl, location } t_cause_of_301;
 
 typedef struct type_session {
