@@ -25,9 +25,7 @@
 #include <sys/time.h>
 #include <sys/stat.h>
 #include <sys/wait.h>
-#ifdef HAVE_SYS_SOCKET_H
 #include <sys/socket.h>
-#endif
 #include "global.h"
 #include "alternative.h"
 #include "libstr.h"
@@ -56,6 +54,8 @@
 #define WAIT_FOR_LOCK          3
 #define FILESIZE_BUFFER_SIZE  30
 
+#define RPROXY_BUFFER_SIZE     4 * KILOBYTE
+
 #define rs_QUIT       -1
 #define rs_DISCONNECT -2
 #define rs_FORCE_QUIT -3
@@ -73,6 +73,7 @@ extern char *fb_alterlist;
 extern char *hs_eol;
 extern char *hs_conlen;
 extern char *hs_contyp;
+extern char *hs_forwarded;
 
 /* Read a file from disk and send it to the client.
  */
@@ -1118,7 +1119,7 @@ static t_access allow_alter(t_session *session) {
 
 	if ((access = ip_allowed(&(session->ip_address), session->host->alter_list)) != allow) {
 		return access;
-	} else if ((x_forwarded_for = get_headerfield("X-Forwarded-For:", session->headerfields)) == NULL) {
+	} else if ((x_forwarded_for = get_headerfield(hs_forwarded, session->headerfields)) == NULL) {
 		return allow;
 	} else if (parse_ip(x_forwarded_for, &forwarded_ip) == -1) {
 		return allow;
@@ -1398,3 +1399,105 @@ int handle_xml_file(t_session *session) {
 	return transform_xml(session);
 }
 #endif
+
+#ifdef ENABLE_RPROXY
+int proxy_request(t_session *session, t_rproxy *rproxy) {
+	t_rproxy_options options;
+	t_rproxy_webserver webserver;
+	char buffer[RPROXY_BUFFER_SIZE + 1], *begin, *end;
+	int bytes_read, bytes_in_buffer = 0, result = 200, code;
+	bool first_line_read = false;
+
+	/* Intialize data structure
+	 */
+	init_rproxy_options(&options, session->client_socket, &(session->ip_address),
+	                    session->method, session->request_uri, session->headerfields,
+	                    session->body, session->content_length);
+
+	session->keep_alive = false;
+
+	/* Connect to webserver
+	 */
+	if ((webserver.socket = connect_to_webserver(rproxy)) == -1) {
+		return 503;
+	}
+
+#ifdef ENABLE_SSL
+	webserver.use_ssl = rproxy->use_ssl;
+	if (webserver.use_ssl) {
+		if (ssl_connect(&(webserver.ssl), &(webserver.ssn), &(webserver.socket)) == -1) {
+			close(webserver.socket);
+			return 503;
+		}
+	}
+#endif
+
+	/* Send request to webserver
+	 */
+	if (send_request_to_webserver(&webserver, &options, rproxy) == -1) {
+		result = -1;
+	}
+
+	/* Read request from webserver and send to client
+	 */
+	while ((bytes_read = read_from_webserver(&webserver, buffer + bytes_in_buffer, RPROXY_BUFFER_SIZE - bytes_in_buffer)) > 0) {
+		if (bytes_read == -1) {
+			if (errno != EINTR) {
+				result = -1;
+				break;
+			}
+			continue;
+		}
+
+		/* Read first line and extract return code
+		 */
+		if (first_line_read == false) {
+			bytes_in_buffer += bytes_read;
+
+			*(buffer + bytes_in_buffer) = '\0';
+			if (strstr(buffer, "\r\n") != NULL) {
+				if ((begin = strchr(buffer, ' ')) != NULL) {
+					begin++;
+					if ((end = strchr(begin, ' ')) != NULL) {
+						*end = '\0';
+						if ((code = str2int(begin)) != -1) {
+							session->return_code = code;
+						}
+						*end = ' ';
+					}
+				}
+
+				first_line_read = true;
+				bytes_read = bytes_in_buffer;
+				bytes_in_buffer = 0;
+			} else if (bytes_in_buffer == RPROXY_BUFFER_SIZE) {
+				result = -1;
+				break;
+			} else {
+				continue;
+			}
+		}
+
+		/* Send data to client
+		 */
+		if (send_buffer(session, buffer, bytes_read) == -1) {
+			result = -1;
+			break;
+		}
+
+		session->data_sent = true;
+	}
+
+	/* Close connection to webserver
+	 */
+#ifdef ENABLE_SSL
+	if (webserver.use_ssl) {
+		ssl_close(&(webserver.ssl));
+	}
+#endif
+
+	close(webserver.socket);
+
+	return result;
+}
+#endif
