@@ -391,7 +391,7 @@ bool toolkit_rules_oke(t_url_toolkit *url_toolkit) {
 static int do_rewrite(char *url, regex_t *regexp, regmatch_t *pmatch, char *rep, char **new_url, int loop) {
 	int ofs, len, len_rep, i, n;
 	char *repl, *c, *sub, *tmp;
-	bool first_run = true, replaced;
+	bool first_run = true;
 
 	if ((url == NULL) || (regexp == NULL) || (rep == NULL) || (new_url == NULL)) {
 		return -1;
@@ -416,7 +416,6 @@ static int do_rewrite(char *url, regex_t *regexp, regmatch_t *pmatch, char *rep,
 		/* Replace '$x' in replacement string with substring.
 		 */
 		c = repl;
-		replaced = false;
 		while (*c != '\0') {
 			if (*c == '$') {
 				if ((*(c+1) >= '0') && (*(c+1) <= '9')) {
@@ -429,7 +428,10 @@ static int do_rewrite(char *url, regex_t *regexp, regmatch_t *pmatch, char *rep,
 						}
 						sub[len] = '\0';
 					} else {
-						sub = strdup("");
+						if ((sub = strdup("")) == NULL) {
+							free(repl);
+							return -1;
+						}
 					}
 					n = c - repl;
 
@@ -438,12 +440,9 @@ static int do_rewrite(char *url, regex_t *regexp, regmatch_t *pmatch, char *rep,
 						return -1;
 					}
 
-					if (replaced) {
-						free(repl);
-					}
+					free(repl);
 					repl = tmp;
 					c = repl + n + strlen(sub) - 1;
-					replaced = true;
 					free(sub);
 				}
 			}
