@@ -23,6 +23,7 @@
 #define UT_ERROR        -1
 #define UT_RETURN        0
 #define UT_EXIT          1
+#define UT_RPROXY        2
 #define UT_REDIRECT    301
 #define UT_DENY_ACCESS 403
 
@@ -31,7 +32,8 @@
 #define IU_ISDIR         2
 
 typedef enum { tc_none, tc_match, tc_requesturi, tc_usessl, tc_oldbrowser } t_toolkit_condition;
-typedef enum { to_none, to_rewrite, to_sub, to_expire, to_skip, to_denyaccess, to_redirect, to_fastcgi, to_ban, to_replace } t_toolkit_operation;
+typedef enum { to_none, to_rewrite, to_sub, to_expire, to_skip, to_denyaccess, to_redirect,
+	to_fastcgi, to_ban, to_replace } t_toolkit_operation;
 typedef enum { tf_continue, tf_return, tf_exit } t_toolkit_flow;
 
 typedef struct type_toolkit_rule {
